import argparse
from typing import Dict, List
from dotenv import load_dotenv
from tqdm import tqdm
import concurrent.futures

from astrapy.db import AstraDB
from cassandra.cluster import Cluster
from cassandra.auth import PlainTextAuthProvider

from vdf_io.constants import INT_MAX
from vdf_io.names import DBNames
from vdf_io.import_vdf.vdf_import_cls import ImportVDB
from vdf_io.meta_types import NamespaceMeta
import re
from vdf_io.util import (
    clean_documents,
    set_arg_from_input,
    set_arg_from_password,
    standardize_metric_reverse,
)

load_dotenv()


class ImportAstraDB(ImportVDB):
    DB_NAME_SLUG = DBNames.ASTRADB

    @classmethod
    def make_parser(cls, subparsers):
        parser_astradb = subparsers.add_parser(
            cls.DB_NAME_SLUG, help="Import data to Datastax Astra DB"
        )
        parser_astradb.add_argument(
            "--endpoint", type=str, help="Location of AstraDB instance"
        )
        parser_astradb.add_argument(
            "--astradb_api_key", type=str, help="AstraDB API key"
        )
        parser_astradb.add_argument(
            "--via_cql",
            type=bool,
            action=argparse.BooleanOptionalAction,
            help="Whether to use CQL to export data",
            default=False,
        )
        parser_astradb.add_argument(
            "--secure_connect_bundle",
            type=str,
            help="Path to the secure connect bundle",
        )

    @classmethod
    def import_vdb(cls, args):
        """
        Import data to Datastax Astra DB
        """
        if args.get("via_cql"):
            set_arg_from_input(
                args,
                "secure_connect_bundle",
                "Enter the path to the secure connect bundle: ",
                str,
            )
            astradb_import = ImportAstraDB(args)
            astradb_import.all_collections = astradb_import.get_all_index_names_cql()
            astradb_import.upsert_data(via_cql=True)
            return astradb_import
        set_arg_from_input(
            args,
            "endpoint",
            "Enter the URL of AstraDB instance (default: value of os.environ['ASTRA_DB_API_ENDPOINT']): ",
            str,
            env_var="ASTRA_DB_API_ENDPOINT",
        )
        set_arg_from_password(
            args,
            "astradb_api_key",
            "Enter the AstraDB API key (default: value of os.environ['ASTRA_DB_APPLICATION_TOKEN']): ",
            "ASTRA_DB_APPLICATION_TOKEN",
        )
        astradb_import = ImportAstraDB(args)
        astradb_import.upsert_data()
        return astradb_import

    def __init__(self, args):
        super().__init__(args)
        if not self.args.get("via_cql"):
            self.db = AstraDB(
                token=self.args.get("astradb_api_key"),
                api_endpoint=self.args.get("endpoint"),
            )
        else:
            self.session = Cluster(
                cloud={"secure_connect_bundle": self.args["secure_connect_bundle"]},
                auth_provider=PlainTextAuthProvider(
                    "token", self.args.get("astradb_api_key")
                ),
            ).connect()
            self.session.execute(
                "USE " + self.args.get("keyspace", "default_keyspace"), timeout=100.0
            )

    def get_all_index_names_cql(self):
        query = f"SELECT * FROM system_schema.tables where keyspace_name='{self.args['keyspace']}'"
        result = self.session.execute_cql(query, timeout=100.0)
        return [f"{row['table_name']}" for row in result]

    def upsert_data(self, via_cql=False):
        self.total_imported_count = 0
        max_hit = False
        indexes_content: Dict[str, List[NamespaceMeta]] = self.vdf_meta["indexes"]
        index_names: List[str] = list(indexes_content.keys())
        if len(index_names) == 0:
            raise ValueError("No indexes found in VDF_META.json")

        # Load Parquet file
        # print(indexes_content[index_names[0]]):List[NamespaceMeta]
        for index_name, index_meta in tqdm(
            indexes_content.items(), desc="Importing indexes"
        ):
            for namespace_meta in tqdm(index_meta, desc="Importing namespaces"):
                self.set_dims(namespace_meta, index_name)
                data_path = namespace_meta["data_path"]
                final_data_path = self.get_final_data_path(data_path)
                new_index_name = index_name + (
                    f'_{namespace_meta["namespace"]}'
                    if namespace_meta["namespace"]
                    else ""
                )
                new_index_name = self.compliant_name(new_index_name)
                if via_cql:
                    collection_list = self.get_all_index_names_cql()
                    tqdm.write(f"Existing collections: {collection_list}")
                else:
                    collection_list = self.db.get_collections()["status"]["collections"]
                new_index_name = self.create_new_name(
                    new_index_name,
                    collection_list,
                    delimiter="_",
                )
                tqdm.write(f"Index name that will be used: {new_index_name}")
                # create collection
                if not via_cql:
                    collection = self.db.create_collection(
                        new_index_name,
                        dimension=namespace_meta["dimensions"],
                        metric=standardize_metric_reverse(
                            namespace_meta.get("distance_metric"),
                            self.DB_NAME_SLUG,
                        ),
                    )
                else:
                    """
                    CREATE TABLE default_keyspace.users (
                        firstname text,
                        lastname text,
                        email text,
                        "favorite color" text,
                        PRIMARY KEY (firstname, lastname)
                    )"""

                    self.session.execute(
                        f"CREATE TABLE IF NOT EXISTS {self.args['keyspace']}.{new_index_name}"
                        f" (id text PRIMARY KEY, \"$vector\" vector<float,{namespace_meta['dimensions']}>)"
                    )
                parquet_files = self.get_parquet_files(final_data_path)
                vectors = {}
                metadata = {}
                for parquet_file in tqdm(parquet_files, desc="Importing parquet files"):
                    (
                        vector_column_names,
                        vector_column_name,
                    ) = self.get_vector_column_name(index_name, namespace_meta)

                    parquet_file_path = self.get_file_path(
                        final_data_path, parquet_file
                    )

                    df = self.read_parquet_progress(parquet_file_path)
                    if len(vectors) > (self.args.get("max_num_rows") or INT_MAX):
                        max_hit = True
                        break
                    if len(vectors) + len(df) > (
                        self.args.get("max_num_rows") or INT_MAX
                    ):
                        df = df.head(
                            (self.args.get("max_num_rows") or INT_MAX) - len(vectors)
                        )
                        max_hit = True
                    self.update_vectors(vectors, vector_column_name, df)
                    self.update_metadata(metadata, vector_column_names, df)
                    if max_hit:
                        break
                self.total_imported_count += self.flush_to_db(
                    vectors, metadata, collection, via_cql=via_cql
                )
            tqdm.write(
                f"Collection {index_name} imported successfully as {new_index_name}"
            )
            tqdm.write(f"{self.total_imported_count} points imported in total.")

        print("Data imported successfully.")
        self.args["imported_count"] = self.total_imported_count

    def flush_to_db(self, vectors, metadata, collection, via_cql, parallel=True):
        if via_cql:
            keys = list(set(vectors.keys()).union(set(metadata.keys())))
            for id in keys:
                self.session.execute(
                    f"INSERT INTO {self.args['keyspace']}.{collection.name} (id, \"$vector\", {', '.join(metadata[id].keys())}) "
                    f"VALUES ('{id}', {vectors[id]}, {', '.join([str(v) for v in metadata[id].values()])})"
                )
            return len(vectors)

        keys = list(set(vectors.keys()).union(set(metadata.keys())))
        if not parallel:
            return flush_to_db_sync(
                collection,
                keys,
                [vectors.get(k) for k in keys],
                [metadata.get(k, {}) for k in keys],
            )

        def flush_batch_to_db(collection, keys, vectors, metadata):
            documents = [
                {"_id": id, "$vector": vector, **metadata}
                for id, vector, metadata in zip(keys, vectors, metadata)
            ]
            # replace nan with None
            clean_documents(documents)
            try:
                response = collection.upsert_many(documents=documents)
                return response
            except Exception as e:
                tqdm.write(f"Error upserting batch: {e}")
                return

        BATCH_SIZE = 20
        num_parallel_threads = self.args.get("parallel", 5) or 5
        total_points = len(keys)
        batches = [
            (
                keys[i : i + BATCH_SIZE],
                [vectors.get(k) for k in keys[i : i + BATCH_SIZE]],
                [metadata.get(k, {}) for k in keys[i : i + BATCH_SIZE]],
            )
            for i in range(0, total_points, BATCH_SIZE)
        ]

        with concurrent.futures.ThreadPoolExecutor(
            max_workers=num_parallel_threads
        ) as executor, tqdm(
            total=total_points,
            desc=f"Flushing to DB in batches of {BATCH_SIZE} in {num_parallel_threads} threads",
        ) as pbar:
            future_to_batch = {
                executor.submit(flush_batch_to_db, collection, *batch): batch
                for batch in batches
            }

            for future in concurrent.futures.as_completed(future_to_batch):
                batch = future_to_batch[future]
                try:
                    future.result()
                    pbar.update(len(batch[0]))
                except Exception as e:
                    tqdm.write(f"Batch upsert failed with error: {e}")

        return total_points

    def compliant_name(self, name):
        return re.sub(r"[- ./]", "_", name)


def flush_to_db_sync(collection, keys, vectors, metadata):
    # in batches of 20 keys using upsert_many
    BATCH_SIZE = 20
    batches = list(range(0, len(keys), BATCH_SIZE))
    for batch in tqdm(batches, desc="Flushing to DB in batches of 20"):
        key_batch = keys[batch : batch + BATCH_SIZE]
        vector_batch = [vectors[k] for k in key_batch]
        metadata_batch = [metadata[k] for k in key_batch]
        upsert_batch(collection, metadata, key_batch, vector_batch, metadata_batch)


def upsert_batch(collection, metadata, key_batch, vector_batch, metadata_batch):
    documents = [
        {"_id": id, "$vector": vector, **metadata}
        for id, vector in zip(key_batch, vector_batch, metadata_batch)
    ]
    # replace nan with None
    clean_documents(documents)

    response = collection.upsert_many(documents=documents)
    # check if any of the elements are errors
    if any([isinstance(r, Exception) for r in response]):
        for i, r in enumerate(response):
            if isinstance(r, Exception):
                tqdm.write(f"Error upserting: {r} for {documents[i]}")



# import os
# from azure.search.documents import SearchClient
# from azure.search.documents.indexes.models import (
#     CorsOptions,
#     ScoringProfile,
#     SearchField,
#     SemanticConfiguration,
#     VectorSearch,
# )
# from azure.core.credentials import AzureKeyCredential
# from azure.core.exceptions import ResourceNotFoundError
# from azure.identity import DefaultAzureCredential, InteractiveBrowserCredential
# from azure.search.documents import SearchClient
# from azure.search.documents.indexes import SearchIndexClient, SearchIndexerClient
# from azure.search.documents.indexes.models import (
#     ExhaustiveKnnAlgorithmConfiguration,
#     ExhaustiveKnnParameters,
#     HnswAlgorithmConfiguration,
#     HnswParameters,
#     SearchIndex,
#     SemanticConfiguration,
#     SemanticField,
#     SemanticPrioritizedFields,
#     SemanticSearch,
#     VectorSearch,
#     VectorSearchAlgorithmKind,
#     VectorSearchAlgorithmMetric,
#     VectorSearchProfile,
# )
# # from langchain_core.utils import get_from_env

# # AZURE_SEARCH_SERVICE_ENDPOINT
# # AZURE_SEARCH_INDEX_NAME
# # AZURE_SEARCH_API_KEY
# # doc count: https://learn.microsoft.com/en-us/python/api/azure-search-documents/azure.search.documents.searchclient?view=azure-python#azure-search-documents-searchclient-get-document-count
# # upsert: https://learn.microsoft.com/en-us/python/api/azure-search-documents/azure.search.documents.searchclient?view=azure-python#azure-search-documents-searchclient-merge-or-upload-documents
# # search, skip: https://learn.microsoft.com/en-us/python/api/azure-search-documents/azure.search.documents.searchclient?view=azure-python#azure-search-documents-searchclient-search
# # FIELDS_ID = get_from_env(
# #     key="AZURESEARCH_FIELDS_ID", env_key="AZURESEARCH_FIELDS_ID", default="id"
# # )
# # FIELDS_CONTENT = get_from_env(
# #     key="AZURESEARCH_FIELDS_CONTENT",
# #     env_key="AZURESEARCH_FIELDS_CONTENT",
# #     default="content",
# # )
# # FIELDS_CONTENT_VECTOR = get_from_env(
# #     key="AZURESEARCH_FIELDS_CONTENT_VECTOR",
# #     env_key="AZURESEARCH_FIELDS_CONTENT_VECTOR",
# #     default="content_vector",
# # )
# # FIELDS_METADATA = get_from_env(
# #     key="AZURESEARCH_FIELDS_TAG", env_key="AZURESEARCH_FIELDS_TAG", default="metadata"
# # )
# index_name = "azuresearch"
# vector_search = VectorSearch(
#     algorithms=[
#         HnswAlgorithmConfiguration(
#             name="default",
#             kind=VectorSearchAlgorithmKind.HNSW,
#             parameters=HnswParameters(
#                 m=4,
#                 ef_construction=400,
#                 ef_search=500,
#                 metric=VectorSearchAlgorithmMetric.COSINE,
#             ),
#         ),
#         ExhaustiveKnnAlgorithmConfiguration(
#             name="default_exhaustive_knn",
#             kind=VectorSearchAlgorithmKind.EXHAUSTIVE_KNN,
#             parameters=ExhaustiveKnnParameters(
#                 metric=VectorSearchAlgorithmMetric.COSINE
#             ),
#         ),
#     ],
#     profiles=[
#         VectorSearchProfile(
#             name="myHnswProfile",
#             algorithm_configuration_name="default",
#         ),
#         VectorSearchProfile(
#             name="myExhaustiveKnnProfile",
#             algorithm_configuration_name="default_exhaustive_knn",
#         ),
#     ],
# )
# key = os.environ.get("AZURE_SEARCH_API_KEY")
# endpoint = os.environ.get("AZURE_SEARCH_SERVICE_ENDPOINT")
# credential = AzureKeyCredential(key) if key else DefaultAzureCredential()
# user_agent = "vector-io"
# # create new index
# index_client: SearchIndexClient = SearchIndexClient(
#     endpoint=endpoint, credential=credential, user_agent=user_agent
# )

# if index_name in [x.name for x in index_client.list_indexes()]:
#     index = index_client.get_index(index_name)
# else:
#     index = SearchIndex(
#         name=index_name,
#         fields=fields,
#         vector_search=vector_search,
#         semantic_search=semantic_search,
#         scoring_profiles=scoring_profiles,
#         default_scoring_profile=default_scoring_profile,
#         cors_options=cors_options,
#     )
#     index_client.create_index(index)
# # upsert data
# search_client: SearchClient = SearchClient(
#     endpoint=endpoint,
#     index_name=index_name,
#     credential=credential,
#     user_agent=user_agent,
# )
# documents = []
# search_client.merge_or_upload_documents(documents=documents)



import json
import os
import sys
from tqdm import tqdm

import chromadb

from vdf_io.constants import DEFAULT_BATCH_SIZE, DISK_SPACE_LIMIT
from vdf_io.names import DBNames
from vdf_io.util import expand_shorthand_path, set_arg_from_input
from vdf_io.export_vdf.vdb_export_cls import ExportVDB


class ExportChroma(ExportVDB):
    DB_NAME_SLUG = DBNames.CHROMA

    @classmethod
    def make_parser(cls, subparsers):
        parser_chroma = subparsers.add_parser(
            cls.DB_NAME_SLUG, help="Export data from Chroma"
        )

        parser_chroma.add_argument(
            "--host_port", type=str, help="Host and port of Chroma instance"
        )
        parser_chroma.add_argument(
            "--persistent_path",
            type=str,
            help="Path to persistent storage for Chroma",
        )
        parser_chroma.add_argument(
            "--collections", type=str, help="Names of collections to export"
        )
        parser_chroma.add_argument(
            "--batch_size",
            type=int,
            help="Batch size for exporting data",
            default=DEFAULT_BATCH_SIZE,
        )

    @classmethod
    def export_vdb(cls, args):
        if args.get("persistent_path") is None:
            set_arg_from_input(
                args,
                "host_port",
                "Enter the host:port of chroma instance: ",
                str,
            )
        if args.get("host_port") is None or args.get("host_port") == "":
            set_arg_from_input(
                args,
                "persistent_path",
                "Enter the path to persistent storage for Chroma: ",
                str,
            )
        if (args.get("host_port") is None or args.get("host_port") == "") and (
            args.get("persistent_path") is None or args.get("persistent_path") == ""
        ):
            set_arg_from_input(
                args,
                "api_key",
                "Enter the API key for Chroma: ",
                str,
            )
            set_arg_from_input(
                args,
                "tenant",
                "Enter the tenant for Chroma: ",
                str,
            )
            set_arg_from_input(
                args,
                "database",
                "Enter the database for Chroma: ",
                str,
            )
        set_arg_from_input(
            args,
            "batch_size",
            f"Enter the batch size for exporting data (default: {DEFAULT_BATCH_SIZE}):",
            int,
            DEFAULT_BATCH_SIZE,
        )
        chroma_export = ExportChroma(args)
        chroma_export.all_collections = chroma_export.get_all_index_names()
        set_arg_from_input(
            args,
            "collections",
            "Enter the name of collection(s) to export (comma-separated) (hit return to export all):",
            str,
            choices=chroma_export.all_collections,
        )
        chroma_export.get_data()
        return chroma_export

    def __init__(self, args):
        super().__init__(args)
        if self.args.get("host_port") is not None:
            host_port = self.args.get("host_port")
            self.client = chromadb.HttpClient(
                host=host_port.split(":")[0], port=int(host_port.split(":")[1])
            )
        elif self.args.get("persistent_path") is not None:
            self.client = chromadb.PersistentClient(
                path=expand_shorthand_path(self.args.get("persistent_path"))
            )
        else:
            self.client = chromadb.CloudClient(
                tenant=self.args.get("tenant"),
                database=self.args.get("database"),
                api_key=self.args.get("api_key"),
            )

    def get_all_index_names(self):
        return [coll.name for coll in self.client.list_collections()]

    def get_index_names(self):
        if self.args.get("collections", None) is not None:
            return self.args["collections"].split(",")
        return self.get_all_index_names()

    def get_data(self):
        batch_size = self.args.get("batch_size") or DEFAULT_BATCH_SIZE
        index_metas = {}
        for i, collection_name in tqdm(
            enumerate(self.get_index_names()), desc="Exporting collections"
        ):
            dims = -1
            col = self.client.get_collection(collection_name)
            vectors_directory = self.create_vec_dir(collection_name)
            existing_count = col.count()
            total = 0
            for j in tqdm(
                range(0, existing_count, batch_size),
                desc=f"Exporting {collection_name} collection",
            ):
                batch = col.get(
                    include=["metadatas", "documents", "embeddings"],
                    limit=batch_size,
                    offset=j,
                )
                vectors = {}
                metadata = {}
                chroma_ids = batch["ids"]
                embeddings = batch["embeddings"]
                chroma_metadatas = batch["metadatas"]
                uris = batch["uris"]
                data = batch["data"]
                documents = batch["documents"]
                dims = len(embeddings[0])
                for idx, chroma_id in enumerate(chroma_ids):
                    vectors[chroma_id] = embeddings[idx]
                    metadata[chroma_id] = chroma_metadatas[idx]
                    metadata[chroma_id]["document"] = documents[idx]
                    if uris is not None and len(uris) > idx:
                        metadata[chroma_id]["uri"] = uris[idx]
                    if data is not None and len(data) > idx:
                        metadata[chroma_id]["data"] = data[idx]
                if sys.getsizeof(vectors) + sys.getsizeof(metadata) > DISK_SPACE_LIMIT:
                    # save_vectors_to_parquet
                    total += self.save_vectors_to_parquet(
                        vectors, metadata, vectors_directory
                    )
            total += self.save_vectors_to_parquet(vectors, metadata, vectors_directory)
            namespace_metas = [
                self.get_namespace_meta(
                    collection_name,
                    vectors_directory,
                    total=existing_count,
                    num_vectors_exported=total,
                    dim=dims,
                    vector_columns=["vector"],
                    distance=(
                        col.metadata.get("hnsw:space", "cosine")
                        if (hasattr(col, "metadata") and col.metadata is not None)
                        else "cosine"
                    ),
                )
            ]
            index_metas[collection_name] = namespace_metas
        self.file_structure.append(os.path.join(self.vdf_directory, "VDF_META.json"))
        internal_metadata = self.get_basic_vdf_meta(index_metas)
        meta_text = json.dumps(internal_metadata.model_dump(), indent=4)
        tqdm.write(meta_text)
        with open(os.path.join(self.vdf_directory, "VDF_META.json"), "w") as json_file:
            json_file.write(meta_text)
        return True



from typing import List
from vdf_io.marqo_vespa_util import VespaClient
from rich import print as rprint

from vdf_io.names import DBNames
from vdf_io.util import set_arg_from_input, set_arg_from_password
from vdf_io.export_vdf.vdb_export_cls import ExportVDB


class ExportVespa(ExportVDB):
    DB_NAME_SLUG = DBNames.VESPA

    @classmethod
    def make_parser(cls, subparsers):
        parser_vespa = subparsers.add_parser(
            cls.DB_NAME_SLUG, help="Export data from Vespa"
        )

        parser_vespa.add_argument(
            "--endpoint", type=str, help="Location of Vespa instance"
        )

        parser_vespa.add_argument(
            "--vespa_cloud_secret_token", type=str, help="Vespa cloud secret token"
        )

        parser_vespa.add_argument(
            "--cert_file", type=str, help="Path to the certificate file"
        )
        parser_vespa.add_argument(
            "--pk_file", type=str, help="Path to the private key file"
        )
        parser_vespa.add_argument("--schemas", type=str, help="")

    @classmethod
    def export_vdb(cls, args):
        set_arg_from_input(
            args,
            "endpoint",
            "Enter the URL of Vespa instance (default: 'http://localhost:8080'): ",
            str,
            None,
            env_var="VESPA_HOST",
        )
        if not args.get("cert_file"):
            set_arg_from_password(
                args,
                "vespa_cloud_secret_token",
                "Enter the Vespa cloud secret token (hit return to enter cert file path instead): ",
                "VESPA_CLOUD_SECRET_TOKEN",
            )
        if args.get("cert_file"):
            set_arg_from_input(
                args,
                "pk_file",
                "Enter the path to the private key file: ",
                str,
                default_value=None,
            )
        if not args["vespa_cloud_secret_token"]:
            set_arg_from_input(
                args,
                "cert_file",
                "Enter the path to the certificate file: ",
                str,
                default_value=None,
            )
        vespa_export = ExportVespa(args)
        vespa_export.get_data()
        return vespa_export

    def __init__(self, args):
        super().__init__(args)

    def get_index_names(self) -> List[str]:
        raise NotImplementedError()  # not available in pyvespa

    def get_data(self):
        schemas = self.args.get("schemas", []).split(",")
        for schema in schemas:
            self.get_data_for_index(schema)
        # Add code here to fetch the data from Vespa

    def get_data_for_index(self, index_name):
        self.vespa_client = VespaClient(
            config_url=self.args["endpoint"],
            query_url=self.args["endpoint"],
            document_url=self.args["endpoint"],
            content_cluster_name=index_name,
            cert_file=self.args.get("cert_file", None),
            pk_file=self.args.get("pk_file", None),
        )
        all_docs = self.vespa_client.get_all_documents(index_name, stream=True)
        rprint(all_docs.document_count)



from typing import Dict, List
from dotenv import load_dotenv
from tqdm import tqdm

import chromadb

from vdf_io.constants import DEFAULT_BATCH_SIZE, INT_MAX
from vdf_io.meta_types import NamespaceMeta
from vdf_io.names import DBNames
from vdf_io.util import (
    cleanup_df,
    divide_into_batches,
    expand_shorthand_path,
    set_arg_from_input,
)
from vdf_io.import_vdf.vdf_import_cls import ImportVDB


load_dotenv()


class ImportChroma(ImportVDB):
    DB_NAME_SLUG = DBNames.CHROMA

    @classmethod
    def make_parser(cls, subparsers):
        """
        Import data to Chroma
        """
        parser_chroma = subparsers.add_parser(
            cls.DB_NAME_SLUG, help="Import data from Chroma"
        )
        parser_chroma.add_argument(
            "--host_port", type=str, help="Host and port of Chroma instance"
        )
        parser_chroma.add_argument(
            "--persistent_path",
            type=str,
            help="Path to persistent storage for Chroma",
        )

    @classmethod
    def import_vdb(cls, args):
        if args.get("persistent_path") is None:
            set_arg_from_input(
                args,
                "host_port",
                "Enter the host:port of chroma instance: ",
                str,
            )
        if args.get("host_port") is None or args.get("host_port") == "":
            set_arg_from_input(
                args,
                "persistent_path",
                "Enter the path to persistent storage for Chroma: ",
                str,
            )
        if (args.get("host_port") is None or args.get("host_port") == "") and (
            args.get("persistent_path") is None or args.get("persistent_path") == ""
        ):
            set_arg_from_input(
                args,
                "api_key",
                "Enter the API key for Chroma: ",
                str,
            )
            set_arg_from_input(
                args,
                "tenant",
                "Enter the tenant for Chroma: ",
                str,
            )
            set_arg_from_input(
                args,
                "database",
                "Enter the database for Chroma: ",
                str,
            )
        chroma_import = ImportChroma(args)
        chroma_import.upsert_data()
        return chroma_import

    def __init__(self, args):
        # call super class constructor
        super().__init__(args)
        if self.args.get("host_port") is not None:
            host_port = self.args.get("host_port")
            self.client = chromadb.HttpClient(
                host=host_port.split(":")[0], port=int(host_port.split(":")[1])
            )
        elif self.args.get("persistent_path") is not None:
            self.client = chromadb.PersistentClient(
                path=expand_shorthand_path(self.args.get("persistent_path"))
            )
        else:
            self.client = chromadb.CloudClient(
                tenant=self.args.get("tenant"),
                database=self.args.get("database"),
                api_key=self.args.get("api_key"),
            )

    def get_all_index_names(self):
        return [coll.name for coll in self.client.list_collections()]

    def upsert_data(self):
        max_hit = False
        self.total_imported_count = 0
        indexes_content: Dict[str, List[NamespaceMeta]] = self.vdf_meta["indexes"]
        index_names: List[str] = list(indexes_content.keys())
        if len(index_names) == 0:
            raise ValueError("No indexes found in VDF_META.json")
        collections = self.get_all_index_names()
        # Load Parquet file
        # print(indexes_content[index_names[0]]):List[NamespaceMeta]
        for index_name, index_meta in tqdm(
            indexes_content.items(), desc="Importing indexes"
        ):
            for namespace_meta in tqdm(index_meta, desc="Importing namespaces"):
                self.set_dims(namespace_meta, index_name)
                data_path = namespace_meta["data_path"]
                final_data_path = self.get_final_data_path(data_path)
                # Load the data from the parquet files
                parquet_files = self.get_parquet_files(final_data_path)

                new_index_name = index_name + (
                    f'_{namespace_meta["namespace"]}'
                    if namespace_meta["namespace"]
                    else ""
                )
                new_index_name = self.create_new_name(new_index_name, collections)
                if new_index_name not in collections:
                    collection = self.client.create_collection(
                        new_index_name,
                    )
                else:
                    collection = self.client.get_collection(new_index_name)
                (
                    vector_column_names,
                    vector_column_name,
                ) = self.get_vector_column_name(index_name, namespace_meta)
                tqdm.write(f"Vector column name: {vector_column_name}")
                if len(vector_column_names) > 1:
                    tqdm.write("Chroma does not support multiple vector columns")
                    tqdm.write(f"Skipping the rest : {vector_column_names[1:]}")
                for file in tqdm(parquet_files, desc="Iterating parquet files"):
                    file_path = self.get_file_path(final_data_path, file)
                    df = self.read_parquet_progress(
                        file_path,
                        max_num_rows=(
                            (self.args.get("max_num_rows") or INT_MAX)
                            - self.total_imported_count
                        ),
                    )
                    df = cleanup_df(df)
                    # if there are additional columns in the parquet file, add them to the table
                    # split in batches
                    BATCH_SIZE = self.args.get("batch_size") or DEFAULT_BATCH_SIZE
                    for batch in tqdm(
                        divide_into_batches(df, BATCH_SIZE),
                        desc="Importing batches",
                        total=len(df) // BATCH_SIZE,
                    ):
                        if self.total_imported_count + len(batch) >= (
                            self.args.get("max_num_rows") or INT_MAX
                        ):
                            batch = batch[
                                : (self.args.get("max_num_rows") or INT_MAX)
                                - self.total_imported_count
                            ]
                            max_hit = True
                        model_map = namespace_meta.get("model_map", {})

                        # filter out rows with empty or None vector column
                        prev_count = len(batch)
                        batch = batch.dropna(subset=[vector_column_name])
                        non_empty_count = len(batch)
                        if prev_count != non_empty_count:
                            tqdm.write(
                                f"Skipped {prev_count - non_empty_count} rows with empty vector column"
                            )
                        ids = [str(x) for x in batch[self.id_column].tolist()]
                        docs = []

                        if (vector_column_name in model_map) and (
                            model_map[vector_column_name].get(
                                "text_column", "NOT_PROVIDED"
                            )
                            != "NOT_PROVIDED"
                        ):
                            text_column = model_map[vector_column_name]["text_column"]
                            docs = batch[text_column].tolist()
                        else:
                            docs = ids

                        embeddings = (
                            batch[vector_column_name]
                            .apply(
                                # if it is iterable, convert to list
                                # else return the value
                                lambda x: list(x) if hasattr(x, "__iter__") else x
                            )
                            .tolist()
                        )
                        metadatas = []
                        for _, row in batch.iterrows():
                            metadata = row.to_dict()
                            metadata.pop(vector_column_name)
                            metadata.pop(self.id_column)
                            # remove values from metadata which are not str, int, float, bool
                            metadata = {
                                k: v
                                for k, v in metadata.items()
                                if isinstance(v, (str, int, float, bool))
                            }
                            metadatas.append(metadata)

                        collection.upsert(
                            ids=ids,
                            embeddings=embeddings,
                            documents=docs,
                            metadatas=metadatas,
                        )

                        self.total_imported_count += len(batch)
                        if max_hit:
                            break
                tqdm.write(
                    f"Imported {self.total_imported_count} rows into {collection.name}"
                )
                tqdm.write(f"New collection size: {collection.count()}")
                if max_hit:
                    break
        print("Data imported successfully")



import argparse
import json
import os
import sys
from typing import Dict, List
from astrapy.db import AstraDB
from tqdm import tqdm
from cassandra.cluster import Cluster
from cassandra.auth import PlainTextAuthProvider
from cassandra.query import SimpleStatement

from vdf_io.constants import DISK_SPACE_LIMIT
from vdf_io.meta_types import NamespaceMeta
from vdf_io.names import DBNames
from vdf_io.util import (
    set_arg_from_input,
    set_arg_from_password,
)
from vdf_io.export_vdf.vdb_export_cls import ExportVDB


class ExportAstraDB(ExportVDB):
    DB_NAME_SLUG = DBNames.ASTRADB

    @classmethod
    def make_parser(cls, subparsers):
        parser_astradb = subparsers.add_parser(
            cls.DB_NAME_SLUG, help="Export data from AstraDB"
        )

        parser_astradb.add_argument(
            "--endpoint", type=str, help="Location of AstraDB instance"
        )
        parser_astradb.add_argument(
            "--astradb_api_key", type=str, help="AstraDB API key"
        )
        parser_astradb.add_argument(
            "--collections", type=str, help="AstraDB tables to export (comma-separated)"
        )
        parser_astradb.add_argument(
            "--distance_metric",
            type=str,
            help="Distance metric to use for the vectors",
            default="cosine",
            choices=[
                "cosine",
                "euclidean",
                "dot_product",
            ],
        )
        parser_astradb.add_argument(
            "--via_cql",
            type=bool,
            action=argparse.BooleanOptionalAction,
            help="Whether to use CQL to export data",
            default=False,
        )
        parser_astradb.add_argument(
            "--secure_connect_bundle",
            type=str,
            help="Path to the secure connect bundle",
        )
        parser_astradb.add_argument(
            "--keyspace",
            type=str,
            help="Keyspace to connect to",
            default="default_keyspace",
        )
        parser_astradb.add_argument(
            "--fetch_size",
            type=int,
            help="Fetch size for CQL queries",
            default=10000,
        )

    @classmethod
    def export_vdb(cls, args):
        set_arg_from_password(
            args,
            "astradb_api_key",
            "Enter the AstraDB API key (default: value of os.environ['ASTRA_DB_APPLICATION_TOKEN']): ",
            "ASTRA_DB_APPLICATION_TOKEN",
        )
        if args.get("via_cql"):
            set_arg_from_input(
                args,
                "secure_connect_bundle",
                "Enter the path to the secure connect bundle: ",
                str,
            )
            astradb_export = ExportAstraDB(args)
            astradb_export.all_collections = astradb_export.get_all_index_names_cql()
            set_arg_from_input(
                args,
                "collections",
                "Enter the name of collections to export (comma-separated, all will be exported by default): ",
                str,
                None,
                choices=astradb_export.all_collections,
            )
            set_arg_from_input(
                args,
                "fetch_size",
                "Enter the fetch size for CQL queries (default: 10000): ",
                int,
                10000,
            )
            astradb_export.get_data_from_cql()
            return astradb_export
        set_arg_from_input(
            args,
            "endpoint",
            "Enter the URL of AstraDB instance (default: value of os.environ['ASTRA_DB_API_ENDPOINT']): ",
            str,
            env_var="ASTRA_DB_API_ENDPOINT",
        )
        astradb_export = ExportAstraDB(args)
        astradb_export.all_collections = astradb_export.get_all_index_names()
        set_arg_from_input(
            args,
            "collections",
            "Enter the name of collections to export (comma-separated, all will be exported by default): ",
            str,
            None,
            choices=astradb_export.all_collections,
        )
        astradb_export.get_data()
        return astradb_export

    def __init__(self, args):
        super().__init__(args)
        if not self.args.get("via_cql"):
            self.db = AstraDB(
                token=self.args.get("astradb_api_key"),
                api_endpoint=self.args.get("endpoint"),
            )
        else:
            self.session = Cluster(
                cloud={"secure_connect_bundle": self.args["secure_connect_bundle"]},
                auth_provider=PlainTextAuthProvider(
                    "token", self.args.get("astradb_api_key")
                ),
            ).connect()
            self.session.execute(
                "USE " + self.args.get("keyspace", "default_keyspace"), timeout=100.0
            )

    def get_all_index_names(self):
        return self.db.get_collections()["status"]["collections"]

    def get_index_names(self):
        if self.args.get("collections", None) is not None:
            return self.args["collections"].split(",")
        return self.db.get_collections()["status"]["collections"]

    def get_all_index_names_cql(self):
        res = self.session.execute(
            f"SELECT * FROM system_schema.tables where keyspace_name='{self.args['keyspace']}'",
            timeout=100.0,
        )
        return [row.table_name for row in res]

    def get_data_from_cql(self):
        index_names = (
            self.get_all_index_names_cql()
            if self.args.get("collections") is None
            else self.args.get("collections").split(",")
        )
        index_metas: Dict[str, List[NamespaceMeta]] = {}
        self.paging_state = None
        for index_name in tqdm(index_names, desc="Fetching indexes"):
            # count rows using execute()
            count_query = f"SELECT COUNT(*) FROM {index_name}"
            count = self.session.execute(count_query, timeout=100.0).one()
            tqdm.write(f"Total rows in {index_name}: {count[0]}")
            tqdm.write(f"Exporting collection: {index_name}")
            namespace_metas = []
            vectors_directory = self.create_vec_dir(index_name)
            pbar = tqdm(desc="Exporting data", unit="documents", total=count[0])
            no_queries_run = True
            exported_count = 0
            vectors = {}
            metadatas = {}
            while no_queries_run or self.paging_state:
                rows = self.execute_select_all_once(index_name)
                rows = [json.loads(r.doc_json) for r in rows]
                no_queries_run = False
                for row in rows:
                    if "vector" in row:
                        vectors[row["_id"]] = row["vector"]
                    elif "$vector" in row:
                        vectors[row["_id"]] = row["$vector"]
                    metadatas[row["_id"]] = {
                        k: v
                        for k, v in row.items()
                        if k not in ["_id", "$vector", "vector"]
                    }
                    pbar.update(1)
                    if (
                        sys.getsizeof(vectors) + sys.getsizeof(metadatas)
                        > DISK_SPACE_LIMIT
                    ):
                        tqdm.write("Flushing to parquet files on disk")
                        exported_count += self.save_vectors_to_parquet(
                            vectors, metadatas, vectors_directory
                        )
            vectors_added = self.save_vectors_to_parquet(
                vectors, metadatas, vectors_directory
            )
            exported_count += vectors_added
            tqdm.write("Flushing to parquet files on disk")
            sample_doc = rows[0]
            if "$vector" in sample_doc:
                dims = len(sample_doc["$vector"])
            elif "vector" in sample_doc:
                dims = len(sample_doc["vector"])
            else:
                dims = -1
            namespace_metas = [
                self.get_namespace_meta(
                    index_name,
                    vectors_directory,
                    total=exported_count,
                    num_vectors_exported=exported_count,
                    dim=dims,
                    vector_columns=["vector"],
                    distance=self.args.get("distance_metric"),
                )
            ]
            index_metas[index_name] = namespace_metas
        self.file_structure.append(os.path.join(self.vdf_directory, "VDF_META.json"))
        internal_metadata = self.get_basic_vdf_meta(index_metas)
        meta_text = json.dumps(internal_metadata.model_dump(), indent=4)
        tqdm.write(meta_text)
        with open(os.path.join(self.vdf_directory, "VDF_META.json"), "w") as json_file:
            json_file.write(meta_text)
        return True

    def execute_select_all_once(self, index_name):
        query = f"SELECT * FROM {index_name}"
        fetch_size = self.args.get("fetch_size", 10000)
        while True:
            try:
                statement = SimpleStatement(query, fetch_size=fetch_size)
                rows = self.session.execute(
                    statement, paging_state=self.paging_state, timeout=100.0
                )
                self.paging_state = rows.paging_state
                break
            except Exception as e:
                tqdm.write(f"Error occurred: {e}. Reducing fetch size by 10%.")
                fetch_size = int(fetch_size * 0.9)  # reduce fetch size by 10%
        self.paging_state = rows.paging_state
        return list(rows)

    def get_data(self):
        index_names = self.get_index_names()
        index_metas: Dict[str, List[NamespaceMeta]] = {}
        for index_name in index_names:
            tqdm.write(f"Exporting collection: {index_name}")
            namespace_metas = []
            vectors_directory = self.create_vec_dir(index_name)
            collection = self.db.collection(index_name)
            next_page_state = None
            tot_docs = 0
            ids = []
            vectors = {}
            metadatas = {}
            pbar = tqdm(desc="Exporting data", unit="documents")
            exported_count = 0
            while True:
                search_results = collection.find(
                    sort=None, options={"pageState": next_page_state}
                )
                tot_docs += len(search_results["data"]["documents"])
                pbar.update(len(search_results["data"]["documents"]))
                next_page_state = search_results["data"]["nextPageState"]
                # search_results["data"]["documents"] is a list of dictionaries containing _id, vector and other fields, which are metadata
                ids += [doc["_id"] for doc in search_results["data"]["documents"]]
                vectors.update(
                    {
                        doc["_id"]: doc["vector"]
                        for doc in search_results["data"]["documents"]
                        if "vector" in doc
                    }
                )
                # exclude _id and vector from metadata
                metadatas.update(
                    {
                        doc["_id"]: {
                            k: v for k, v in doc.items() if k not in ["_id", "vector"]
                        }
                        for doc in search_results["data"]["documents"]
                    }
                )
                # if getsizeof of vectors and metadata is too large, save to parquet
                if sys.getsizeof(vectors) + sys.getsizeof(metadatas) > DISK_SPACE_LIMIT:
                    tqdm.write("Flushing to parquet files on disk")
                    exported_count += self.save_vectors_to_parquet(
                        vectors, metadatas, vectors_directory
                    )
                if search_results["data"]["nextPageState"] is None:
                    break
            exported_count += self.save_vectors_to_parquet(
                vectors, metadatas, vectors_directory
            )
            sample_doc = collection.find_one()["data"]["document"]
            if "$vector" in sample_doc:
                dims = len(sample_doc["$vector"])
            else:
                dims = -1
            namespace_metas = [
                self.get_namespace_meta(
                    index_name,
                    vectors_directory,
                    total=tot_docs,
                    num_vectors_exported=exported_count,
                    dim=dims,
                    vector_columns=["vector"],
                    distance=self.args.get("distance_metric"),
                )
            ]
            index_metas[index_name] = namespace_metas
        self.file_structure.append(os.path.join(self.vdf_directory, "VDF_META.json"))
        internal_metadata = self.get_basic_vdf_meta(index_metas)
        meta_text = json.dumps(internal_metadata.model_dump(), indent=4)
        tqdm.write(meta_text)
        with open(os.path.join(self.vdf_directory, "VDF_META.json"), "w") as json_file:
            json_file.write(meta_text)
        # print internal metadata properly
        return True



import ssl
from typing import Any, Dict, List, Optional
import httpx
from pydantic import BaseModel, Field
from rich import print as rprint


class Document(BaseModel):
    id: str
    fields: Dict[str, Any]


class VisitDocumentsResponse(BaseModel):
    path_id: str = Field(alias="pathId")
    documents: List[Document]
    document_count: int = Field(alias="documentCount")
    continuation: Optional[str]


class VespaClient:
    def __init__(
        self,
        config_url: str,
        document_url: str,
        query_url: str,
        content_cluster_name: str,
        cert_file: Optional[str] = None,
        pk_file: Optional[str] = None,
        pool_size: int = 10,
        feed_pool_size: int = 10,
        get_pool_size: int = 10,
        delete_pool_size: int = 10,
        partial_update_pool_size: int = 10,
    ):
        """
        Create a VespaClient object.
        Args:
            config_url: Vespa Deploy API base URL
            document_url: Vespa Document API base URL
            query_url: Vespa Query API base URL
            pool_size: Number of connections to keep in the connection pool
            feed_pool_size: Number of connections to keep in batch feed requests connection pool to Vespa
            get_pool_size: Number of connections to keep in batch get requests connection pool to Vespa
            delete_pool_size: Number of connections to keep batch delete requests connection pool to Vespa
            partial_update_pool_size: Number of connections to keep batch partial update requests connection pool to Vespa
        """
        self.config_url = config_url.strip("/")
        self.document_url = document_url.strip("/")
        self.query_url = query_url.strip("/")
        try:
            self.http_client = httpx.Client(
                limits=httpx.Limits(
                    max_keepalive_connections=pool_size, max_connections=pool_size
                ),
                verify=True if cert_file else False,
                cert=(cert_file, pk_file) if cert_file else None,
            )
        except ssl.SSLError as e:  # noqa: F821
            raise VespaError("Failed to create http client due to SSL error", cause=e)
        self.content_cluster_name = content_cluster_name
        self.feed_pool_size = feed_pool_size
        self.get_pool_size = get_pool_size
        self.delete_pool_size = delete_pool_size
        self.partial_pool_size = partial_update_pool_size

    def get_all_documents(
        self, schema: str, stream=False, continuation: Optional[str] = None
    ) -> VisitDocumentsResponse:
        """
        Get all documents in a schema.
        Args:
            schema: Schema to get from
            stream: Whether to stream the response
            continuation: Continuation token for pagination

        Returns:
            BatchGetDocumentResponse object
        """
        try:
            query_params = {
                "stream": str(stream).lower(),
                "continuation": continuation,
            }
            query_string = "&".join(
                [f"{key}={value}" for key, value in query_params.items() if value]
            )
            url = f"{self.document_url}/document/v1/{schema}/{schema}/docid"
            url = f'{url.strip("?")}?{query_string}'
            print(f"{url=}")
            resp = self.http_client.get(url)
        except httpx.HTTPError as e:
            raise VespaError(e) from e
        self._raise_for_status(resp)
        rprint(resp.json())
        return VisitDocumentsResponse(**resp.json())

    def _raise_for_status(self, resp: httpx.Response) -> None:
        try:
            resp.raise_for_status()
        except httpx.HTTPStatusError as e:
            response = e.response
            try:
                json = response.json()
                error_code = json["error-code"]
                message = json["message"]
            except Exception:
                raise VespaStatusError(message=response.text, cause=e) from e

            self._raise_for_error_code(error_code, message, e)


class MarqoErrorMeta(type):
    """
    This metaclass adds a default __init__ method that takes an optional `message: str` and
    an optional `cause: Exception` to subclasses of `MarqoError`.
    """

    def __new__(cls, name, bases, attrs):
        if name != "MarqoError":
            if not any(issubclass(base, MarqoError) for base in bases):
                raise TypeError(
                    f"Class {name} must inherit from {MarqoError.__name__}. "
                    f"Do not use this metaclass directly. Inherit from {MarqoError.__name__} instead."
                )

        return super().__new__(cls, name, bases, attrs)

    def __init__(cls, name, bases, attrs):
        if "__init__" not in attrs:

            def __init__(
                self, message: Optional[str] = None, cause: Optional[Exception] = None
            ):
                super(cls, self).__init__(message, cause)

            setattr(cls, "__init__", __init__)
        super().__init__(name, bases, attrs)


class MarqoError(Exception, metaclass=MarqoErrorMeta):
    """
    Base class for all Marqo errors.
    """

    def __init__(
        self, message: Optional[str] = None, cause: Optional[Exception] = None
    ):
        super().__init__(message)
        self.message = message
        self.cause = cause


class VespaError(MarqoError):
    pass


class VespaStatusError(VespaError):
    @property
    def status_code(self) -> int:
        try:
            return self.cause.response.status_code
        except Exception as e:
            raise Exception(f"Could not get status code from {self.cause}") from e

    def __str__(self) -> str:
        try:
            return f"{self.status_code}: {self.message}"
        except Exception:
            return super().__str__()



import os

from tqdm import tqdm
import weaviate

from vdf_io.export_vdf.vdb_export_cls import ExportVDB
from vdf_io.names import DBNames
from vdf_io.util import set_arg_from_input, set_arg_from_password

# Set these environment variables
URL = os.getenv("YOUR_WCS_URL")
APIKEY = os.getenv("YOUR_WCS_API_KEY")


class ExportWeaviate(ExportVDB):
    DB_NAME_SLUG = DBNames.WEAVIATE

    @classmethod
    def make_parser(cls, subparsers):
        parser_weaviate = subparsers.add_parser(
            cls.DB_NAME_SLUG, help="Export data from Weaviate"
        )

        parser_weaviate.add_argument("--url", type=str, help="URL of Weaviate instance")
        parser_weaviate.add_argument("--api_key", type=str, help="Weaviate API key")
        parser_weaviate.add_argument(
            "--classes", type=str, help="Classes to export (comma-separated)"
        )

    @classmethod
    def export_vdb(cls, args):
        set_arg_from_input(
            args,
            "url",
            "Enter the URL of Weaviate instance: ",
            str,
        )
        set_arg_from_password(
            args,
            "api_key",
            "Enter the Weaviate API key: ",
            "WEAVIATE_API_KEY",
        )
        weaviate_export = ExportWeaviate(args)
        weaviate_export.all_classes = list(
            weaviate_export.client.collections.list_all().keys()
        )
        set_arg_from_input(
            weaviate_export.args,
            "classes",
            "Enter the name of the classes to export (comma-separated, all will be exported by default): ",
            str,
            choices=weaviate_export.all_classes,
        )
        weaviate_export.get_data()
        return weaviate_export

    # Connect to a WCS instance
    def __init__(self, args):
        super().__init__(args)
        self.client = weaviate.connect_to_wcs(
            cluster_url=self.args["url"],
            auth_credentials=weaviate.auth.AuthApiKey(self.args["api_key"]),
            skip_init_checks=True,
        )

    def get_index_names(self):
        if self.args.get("classes") is None:
            return self.all_classes
        else:
            input_classes = self.args["classes"].split(",")
            if set(input_classes) - set(self.all_classes):
                tqdm.write(
                    f"These classes are not present in the Weaviate instance: {set(input_classes) - set(self.all_classes)}"
                )
            return [c for c in self.all_classes if c in input_classes]

    def get_data(self):
        # Get all objects of a class
        index_names = self.get_index_names()
        for class_name in index_names:
            collection = self.client.collections.get(class_name)
            response = collection.aggregate.over_all(total_count=True)
            print(f"{response.total_count=}")

            # objects = self.client.query.get(
            #     wvq.Objects(wvq.Class(class_name)).with_limit(1000)
            # )
            # print(objects)



from vdf_io.names import DBNames
from vdf_io.util import set_arg_from_input
from vdf_io.export_vdf.vdb_export_cls import ExportVDB


class ExportTxtai(ExportVDB):
    DB_NAME_SLUG = DBNames.TXTAI

    @classmethod
    def make_parser(cls, subparsers):
        parser_txtai = subparsers.add_parser(
            cls.DB_NAME_SLUG, help="Export data from Txtai"
        )

        parser_txtai.add_argument(
            "--endpoint", type=str, help="Location of Txtai instance"
        )

    @classmethod
    def export_vdb(cls, args):
        set_arg_from_input(
            args,
            "endpoint",
            "Enter the URL of Txtai instance (default: 'http://localhost:8080'): ",
            str,
            "http://localhost:8080",
        )
        txtai_export = ExportTxtai(args)
        txtai_export.get_data()
        return txtai_export

    def __init__(self, args):
        super().__init__(args)

    def get_data(self):
        pass



from dotenv import load_dotenv


from vdf_io.names import DBNames
from vdf_io.util import (
    set_arg_from_input,
    set_arg_from_password,
)
from vdf_io.import_vdf.vdf_import_cls import ImportVDB


load_dotenv()


class ImportAzureAI(ImportVDB):
    DB_NAME_SLUG = DBNames.AZUREAI

    @classmethod
    def import_vdb(cls, args):
        """
        Import data to Azure AI
        """
        set_arg_from_input(
            args,
            "endpoint",
            "Enter the URL of Azure AI instance (default: '~/.azureai'): ",
            str,
            default_value="~/.azureai",
        )
        set_arg_from_password(
            args,
            arg_name="azureai_api_key",
            prompt="Enter the Azure AI API key (default: value of os.environ['AZUREAI_API_KEY']): ",
            env_var_name="AZUREAI_API_KEY",
        )
        azureai_import = ImportAzureAI(args)
        azureai_import.upsert_data()
        return azureai_import

    @classmethod
    def make_parser(cls, subparsers):
        parser_azureai = subparsers.add_parser(
            cls.DB_NAME_SLUG, help="Import data to azureai"
        )
        parser_azureai.add_argument(
            "--endpoint", type=str, help="Location of Azure AI instance"
        )
        parser_azureai.add_argument(
            "--azureai_api_key", type=str, help="Azure AI API key"
        )
        parser_azureai.add_argument(
            "--tables", type=str, help="Azure AI tables to export (comma-separated)"
        )

    def __init__(self, args):
        super().__init__(args)
        self.endpoint = args.endpoint
        self.azureai_api_key = args.azureai_api_key
        self.tables = args.tables.split(",") if args.tables else []

    def upsert_data(self):
        """
        Upsert data to Azure AI
        """
        print("Upserting data to Azure AI")
